@@ -48,6 +48,24 @@ package inc::MBX::Alien::FLTK::Platform::Unix::Darwin;
         $self->notes('define')->{'__APPLE_COCOA__'}  = 1;    # Alpha
         return 1;
     }
+
+    sub _configure_ar {
+        my $s = shift;
+        print 'Locating library archiver... ';
+        open(my ($OLDOUT), ">&STDOUT");
+        close *STDOUT;
+        my ($ar) = grep { defined $_ } can_run($Config{'gar'}),
+            can_run($Config{'ar'});
+        open(*STDOUT, '>&', $OLDOUT)
+            || exit !print "Couldn't restore STDOUT: $!\n";
+        if (!$ar) {
+            print "Could not find the library archiver, aborting.\n";
+            exit 0;
+        }
+        $ar .= ' -r -c' . (can_run($Config{'ranlib'}) ? ' -s' : '');
+        $s->notes(AR => $ar);
+        print "$ar\n";
+    }
     1;
 }
 
